﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common.Dynamics.GP.eConnect;

namespace Microsoft.Dynamics.GP.eConnect.Serialization
{
    public static class SOPTransactionTypeExtensions
    {
        //DEFEXTPRICE = 1, lets dynamics do it automatically
        public static decimal CalculateSubTotal(this SOPTransactionType sopTransType)
        {
            if (sopTransType == null) throw new ArgumentNullException("sopTransType");
            if (sopTransType.taSopHdrIvcInsert == null) throw new ArgumentNullException("taSopHdrIvcInsert");
            if (sopTransType.taSopLineIvcInsert_Items == null) throw new ArgumentNullException("taSopLineIvcInsert_Items");
            if (sopTransType.taSopLineIvcInsert_Items.Count() < 1) throw new NullReferenceException("taSopLineIvcInsert_Items");

            decimal subTotal = 0;

            foreach (taSopLineIvcInsert_ItemsTaSopLineIvcInsert lineItem in sopTransType.taSopLineIvcInsert_Items)
                subTotal += lineItem.XTNDPRCE;

            //sopTransType.taSopHdrIvcInsert.SUBTOTAL = subTotal;

            return subTotal;
        }

        public static void GenerateSubTotal(this SOPTransactionType sopTransType)
        {
            if (sopTransType == null) throw new ArgumentNullException("sopTransType");
            if (sopTransType.taSopHdrIvcInsert == null) throw new ArgumentNullException("taSopHdrIvcInsert");
            if (sopTransType.taSopLineIvcInsert_Items == null) throw new ArgumentNullException("taSopLineIvcInsert_Items");
            if (sopTransType.taSopLineIvcInsert_Items.Count() < 1) throw new NullReferenceException("taSopLineIvcInsert_Items");

            sopTransType.taSopHdrIvcInsert.SUBTOTAL = CalculateSubTotal(sopTransType);
        }

        public static void GenerateXTNDPRCE(this SOPTransactionType sopTransType)
        {
            if (sopTransType == null) throw new ArgumentNullException("sopTransType");
            if (sopTransType.taSopHdrIvcInsert == null) throw new ArgumentNullException("taSopHdrIvcInsert");
            if (sopTransType.taSopLineIvcInsert_Items == null) throw new ArgumentNullException("taSopLineIvcInsert_Items");
            if (sopTransType.taSopLineIvcInsert_Items.Count() < 1) throw new NullReferenceException("taSopLineIvcInsert_Items");

            for (int i = 0; i < sopTransType.taSopLineIvcInsert_Items.Count(); i++)
                sopTransType.taSopLineIvcInsert_Items[i].XTNDPRCE = sopTransType.taSopLineIvcInsert_Items[i].UNITPRCE * sopTransType.taSopLineIvcInsert_Items[i].QUANTITY;
        }

        public static decimal CalculateTAXAMNT(this SOPTransactionType sopTransType)
        {
            if (sopTransType == null) throw new ArgumentNullException("sopTransType");
            if (sopTransType.taSopHdrIvcInsert == null) throw new ArgumentNullException("taSopHdrIvcInsert");
            if (sopTransType.taSopLineIvcInsert_Items == null) throw new ArgumentNullException("taSopLineIvcInsert_Items");
            if (sopTransType.taSopLineIvcInsert_Items.Count() < 1) throw new NullReferenceException("taSopLineIvcInsert_Items");

            decimal TAXAMNT = 0;

            foreach (taSopLineIvcInsert_ItemsTaSopLineIvcInsert lineItem in sopTransType.taSopLineIvcInsert_Items)
                TAXAMNT += lineItem.TAXAMNT;

            return TAXAMNT;
        }

        //Subtotal+Freight+Misc+TaxAmt+FreightTax+MiscTax-TradeDisc) = DOCATM
        public static decimal CalculateDOCAMT(this SOPTransactionType sopTransType)
        {
            if (sopTransType == null) throw new ArgumentNullException("sopTransType");
            if (sopTransType.taSopHdrIvcInsert == null) throw new ArgumentNullException("taSopHdrIvcInsert");
            if (sopTransType.taSopLineIvcInsert_Items == null) throw new ArgumentNullException("taSopLineIvcInsert_Items");
            if (sopTransType.taSopLineIvcInsert_Items.Count() < 1) throw new NullReferenceException("taSopLineIvcInsert_Items");

            decimal DOCAMT = (sopTransType.taSopHdrIvcInsert.SUBTOTAL +
                              sopTransType.taSopHdrIvcInsert.FREIGHT +
                              sopTransType.taSopHdrIvcInsert.MISCAMNT +
                              CalculateTAXAMNT(sopTransType) +
                              sopTransType.taSopHdrIvcInsert.FRTTXAMT +
                              sopTransType.taSopHdrIvcInsert.MSCTXAMT -
                              sopTransType.taSopHdrIvcInsert.TRDISAMT);

            return DOCAMT;
        }

        public static void GenerateDOCAMT(this SOPTransactionType sopTransType)
        {
            if (sopTransType == null) throw new ArgumentNullException("sopTransType");
            if (sopTransType.taSopHdrIvcInsert == null) throw new ArgumentNullException("taSopHdrIvcInsert");
            if (sopTransType.taSopLineIvcInsert_Items == null) throw new ArgumentNullException("taSopLineIvcInsert_Items");
            if (sopTransType.taSopLineIvcInsert_Items.Count() < 1) throw new NullReferenceException("taSopLineIvcInsert_Items");

            sopTransType.taSopHdrIvcInsert.TAXAMNT = CalculateTAXAMNT(sopTransType);
        }

        public static void GenerateTAXAMNT(this SOPTransactionType sopTransType)
        {
            if (sopTransType == null) throw new ArgumentNullException("sopTransType");
            if (sopTransType.taSopHdrIvcInsert == null) throw new ArgumentNullException("taSopHdrIvcInsert");
            if (sopTransType.taSopLineIvcInsert_Items == null) throw new ArgumentNullException("taSopLineIvcInsert_Items");
            if (sopTransType.taSopLineIvcInsert_Items.Count() < 1) throw new NullReferenceException("taSopLineIvcInsert_Items");

            sopTransType.taSopHdrIvcInsert.TAXAMNT = CalculateTAXAMNT(sopTransType);
        }

        public static decimal CalculateDebitAmount(this SOPTransactionType sopTransType)
        {
            if (sopTransType == null) throw new ArgumentNullException("sopTransType");
            if (sopTransType.taSopDistribution_Items == null) throw new ArgumentNullException("taSopDistribution_Items");
            if (sopTransType.taSopDistribution_Items.Count() < 1) throw new NullReferenceException("taSopDistribution_Items");

            decimal decSubTotal = 0;

            for (int i = 0; i < sopTransType.taSopDistribution_Items.Count(); i++)
            {
                switch (sopTransType.taSopDistribution_Items[i].DISTTYPE.ToType<DistTypes>())
                {
                    case DistTypes.SALES:
                    case DistTypes.TAXES:
                    //case DistTypes.FREIGHT:
                        decSubTotal += sopTransType.taSopDistribution_Items[i].CRDTAMNT;
                        break;
                    default:
                        continue;
                }
            }

            return decSubTotal;
        }
    }
}
