﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Script.Serialization;
using System.Runtime.Serialization.Json;
using System.IO;
using System.Xml.Serialization;


namespace System
{
    public static class StringExtension
    {
        /// <summary>
        /// Takes a xml string and deserializes it back to a object of a type.
        /// </summary>
        /// <typeparam name="T">The type of the members of values.</typeparam>
        /// <param name="strValue">Current String</param>
        /// <returns>returns a deserialized object of the type of T.</returns>
        public static T FromXML<T>(this string strValue)
        {
            using (TextReader textReader = new StringReader(strValue))
            {
                XmlSerializer xmlSerializer = new XmlSerializer(typeof(T));
                return xmlSerializer.Deserialize(textReader).ToType<T>();
            }
        }

        /// <summary>
        /// Takes a JSON string and deserializes it back to a object of a type.
        /// </summary>
        /// <typeparam name="T">The type of the members of values.</typeparam>
        /// <param name="strValue">Current String</param>
        /// <returns>return a deserialized object of the type of T.</returns>
        public static T FromJSON<T>(this string strValue)
        {
            JavaScriptSerializer serializer = new JavaScriptSerializer();

            return serializer.Deserialize<T>(strValue);
        }

        /// <summary>
        /// Takes a JSON2 string and deserializes it back to a object of a type.
        /// </summary>
        /// <typeparam name="T">The type of the members of values.</typeparam>
        /// <param name="strValue">Current String</param>
        /// <returns>returns a deserialized object of the type of T.</returns>
        public static T FromJSON2<T>(this string strValue)
        {
            DataContractJsonSerializer json = new DataContractJsonSerializer(typeof(T));

            using (MemoryStream ms = new MemoryStream())
            {
                ms.Write(Encoding.UTF8.GetBytes(strValue), 0, strValue.Length);

                return json.ReadObject(ms).ToType<T>();
            }
        }

        /// <summary>
        /// Takes a string and encodes it to a Base64 string.
        /// </summary>
        /// <param name="strValue">Current String</param>
        /// <returns>Base64 encode string.</returns>
        public static string ToBase64(this string strValue)
        {
            return Convert.ToBase64String(Encoding.UTF8.GetBytes(strValue));
        }

        /// <summary>
        /// Takes a Base64 string and decodes it to a string.
        /// </summary>
        /// <param name="strValue">Current String</param>
        /// <returns>string of the decoded string.</returns>
        public static string FromBase64(this string strValue)
        {
            return Encoding.UTF8.GetString(Convert.FromBase64String(strValue));
        }
    }
}
